import {
  ForbiddenException,
  forwardRef,
  Inject,
  Injectable,
  NotFoundException,
} from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { FindManyOptions, FindOptionsWhere, Like, Repository } from 'typeorm';
import { Carrito } from './entity/carrito.entity';
import { FullInformacionCarritoView } from './entity/views/full-informacion-carrito.view';
import { InformacionCarritoView } from './entity/views/informacion-carrito.view';
import { Modulo } from '../modulo/entity/modulo.entity';
import { Operador } from '../operador/entity/operador.entity';
import { TipoCarrito } from '../institucion-tipo-carrito/entity/tipo-carrito.entity';
import { CarritoMotivoService } from '../carrito-motivo/carrito-motivo.service';
import { InstitucionService } from '../institucion/institucion.service';
import { InstitucionTipoCarritoService } from '../institucion-tipo-carrito/institucion-tipo-carrito.service';
import { MarcaService } from '../marca/marca.service';
import { ModeloService } from '../modelo/modelo.service';
import { ModuloService } from '../modulo/modulo.service';

@Injectable()
export class CarritoService {
  constructor(
    @InjectRepository(Carrito) private repository: Repository<Carrito>,
    @InjectRepository(FullInformacionCarritoView)
    private fullInformacionCarritoView: Repository<FullInformacionCarritoView>,
    @InjectRepository(InformacionCarritoView)
    private informacionCarritoView: Repository<InformacionCarritoView>,
    @Inject(forwardRef(() => CarritoMotivoService))
    private caritoMotivoService: CarritoMotivoService,
    private institucionService: InstitucionService,
    private institucionTipoCarritoService: InstitucionTipoCarritoService,
    private marcaService: MarcaService,
    private modeloService: ModeloService,
    private moduloService: ModuloService,
  ) {}

  async create(
    admin: Operador,
    carrito: string,
    id_modulo: number,
    id_tipo_carrito: number,
    id_marca: number,
    id_modelo: number,
  ): Promise<{ message: string }> {
    const modulo = await this.moduloService.findById(id_modulo);
    const tipoCarrito =
      await this.institucionTipoCarritoService.findTipoCarritoById(
        id_tipo_carrito,
      );
    const marca = await this.marcaService.findById(id_marca, 'c');
    const modelo = await this.modeloService.findById(id_modelo, 'c');

    // Validamos que el módulo enviado pertenezca a la institutción del admin
    if (admin.institucion.id_institucion != modulo.institucion.id_institucion)
      throw new ForbiddenException(
        'No puedes crear carritos para este módulo porque no pertenece a tu institución.',
      );
    // Creamos y guardamos un registro
    return this.repository
      .save(
        this.repository.create({ carrito, marca, modelo, modulo, tipoCarrito }),
      )
      .then((_) => ({
        message: 'Se creó correctamente un nuevo carrito para este módulo.',
      }));
  }

  async findAll(
    operador: Operador,
    filtros: {
      pagina?: string;
      activo?: string;
      carrito?: string;
      id_institucion?: string;
      id_marca?: string;
      id_modelo?: string;
      id_modulo?: string;
      id_tipo_carrito?: string;
    },
  ): Promise<Carrito[] | [Carrito[], number]> {
    const carritos: Carrito[] = [];
    const institucion = filtros.id_institucion
      ? await this.institucionService.findById(parseInt(filtros.id_institucion))
      : null;
    const marca = filtros.id_marca
      ? await this.marcaService.findById(parseInt(filtros.id_marca), 'c')
      : null;
    const modelo = filtros.id_modelo
      ? await this.modeloService.findById(parseInt(filtros.id_modelo), 'c')
      : null;
    const modulo = filtros.id_modulo
      ? await this.moduloService.findById(parseInt(filtros.id_modulo))
      : null;
    const tipoCarrito = filtros.id_tipo_carrito
      ? await this.institucionTipoCarritoService.findTipoCarritoById(
          parseInt(filtros.id_tipo_carrito),
        )
      : null;
    const options: FindManyOptions<FullInformacionCarritoView> = {};
    const busqueda: FindOptionsWhere<FullInformacionCarritoView> = {};

    if (filtros.activo === 'true') busqueda.activo = 1;
    if (filtros.activo === 'false') busqueda.activo = 0;
    if (filtros.carrito) busqueda.carrito = Like(`%${filtros.carrito}%`);
    if (operador.institucion)
      busqueda.id_institucion = operador.institucion.id_institucion;
    else if (institucion) busqueda.id_institucion = institucion.id_institucion;
    if (marca) busqueda.id_marca = marca.id_marca;
    if (modelo) busqueda.id_modelo = modelo.id_modelo;
    if (modulo) busqueda.id_modulo = modulo.id_modulo;
    if (tipoCarrito) busqueda.id_tipo_carrito = tipoCarrito.id_tipo_carrito;
    options.where = busqueda;
    // Si los filtros tienen el atributo página significa que requiere paginación
    if (filtros.pagina) {
      options.skip = (parseInt(filtros.pagina) - 1) * 25;
      options.take = 25;
      return this.fullInformacionCarritoView
        .findAndCount(options)
        .then((infoCarritos) => {
          for (let i = 0; i < infoCarritos[0].length; i++)
            carritos.push(this.fullViewToCarrito(infoCarritos[0][i]));
          return [carritos, infoCarritos[1]];
        });
    } else
      return this.fullInformacionCarritoView
        .find(options)
        .then((infoCarritos) => {
          for (let i = 0; i < infoCarritos.length; i++)
            carritos.push(this.fullViewToCarrito(infoCarritos[i]));
          return carritos;
        });
  }

  findById(id_carrito: number): Promise<Carrito> {
    return this.informacionCarritoView
      .findOne({ where: { id_carrito } })
      .then((infoCarrito) => {
        if (!infoCarrito)
          throw new NotFoundException('No existe este id carrito.');
        return this.viewToCarrito(infoCarrito);
      });
  }

  findCarrito(
    modulo: Modulo,
    tipoCarrito: TipoCarrito,
    carrito: string,
  ): Promise<Carrito> {
    return this.informacionCarritoView
      .findOne({
        where: {
          carrito,
          id_modulo: modulo.id_modulo,
          id_tipo_carrito: tipoCarrito.id_tipo_carrito,
        },
      })
      .then((infoCarrito) => this.viewToCarrito(infoCarrito));
  }

  findFullInfoById(operador: Operador, id_carrito: number): Promise<Carrito> {
    return this.fullInformacionCarritoView
      .findOne({ where: { id_carrito } })
      .then((infoCarrito) => {
        if (!infoCarrito)
          throw new NotFoundException('No existe este id carrito.');
        if (operador.institucion.id_institucion != infoCarrito.id_institucion)
          throw new ForbiddenException(
            'No puedes acceder a esta información porque este carrito no pertenece a tu institución.',
          );
        return this.fullViewToCarrito(infoCarrito);
      });
  }

  private fullViewToCarrito(infoCarrito: FullInformacionCarritoView): Carrito {
    return this.repository.create({
      id_carrito: infoCarrito.id_carrito,
      activo: infoCarrito.activo === 1,
      carrito: infoCarrito.carrito,
      marca: {
        id_marca: infoCarrito.id_marca,
        marca: infoCarrito.marca,
      },
      modelo: {
        id_modelo: infoCarrito.id_modelo,
        modelo: infoCarrito.modelo,
      },
      modulo: {
        id_modulo: infoCarrito.id_modulo,
        modulo: infoCarrito.modulo,
        institucion: {
          id_institucion: infoCarrito.id_institucion,
          institucion: infoCarrito.institucion,
        },
      },
      tipoCarrito: {
        id_tipo_carrito: infoCarrito.id_tipo_carrito,
        tipo_carrito: infoCarrito.tipo_carrito,
      },
    });
  }

  async update(
    operador: Operador,
    attrs: Partial<Carrito>,
    motivo?: string,
    laboratorioMovil?: boolean,
    numero_equipos?: number,
  ): Promise<{ message: string }> {
    return this.findById(attrs.id_carrito)
      .then((carrito) => {
        // Validamos que el carrito pertenezca a la institución del operador
        if (
          operador.institucion.id_institucion !=
          carrito.modulo.institucion.id_institucion
        )
          throw new ForbiddenException(
            'No puedes modificar la información este carrito porque no pertenece a tu institución.',
          );
        // Validamos que se mande el motivo de desactivación
        if (attrs.activo === false && !motivo)
          throw new ForbiddenException(
            'No se mandó el motivo de la desactivación.',
          );
        // Actualizamos el carrito con los valores enviados del client
        Object.assign(carrito, attrs);
        // Guardamos cambios
        return this.repository.save(carrito);
      })
      .then(async (carrito) => {
        // Creamos reporte de desactivación si se paso a inactivo
        if (attrs.activo === false)
          await this.caritoMotivoService.create(
            carrito,
            operador,
            motivo,
            laboratorioMovil,
            numero_equipos,
          );
        return { message: 'Se guardaron los cambios correctamente.' };
      });
  }

  private viewToCarrito(infoCarrito: InformacionCarritoView): Carrito {
    if (!infoCarrito) return null;
    return this.repository.create({
      id_carrito: infoCarrito.id_carrito,
      modulo: {
        id_modulo: infoCarrito.id_modulo,
        institucion: { id_institucion: infoCarrito.id_institucion },
      },
      tipoCarrito: { id_tipo_carrito: infoCarrito.id_tipo_carrito },
    });
  }
}
