import {
  ConflictException,
  Injectable,
  NotFoundException,
} from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { Not, Repository } from 'typeorm';
import { Infraccion } from './entity/infraccion.entity';
import { Institucion } from '../institucion/entity/institucion.entity';
import { InstitucionInfraccion } from './entity/institucion-infraccion.entity';
import { Operador } from '../operador/entity/operador.entity';
import { InstitucionService } from '../institucion/institucion.service';

@Injectable()
export class InstitucionInfraccionService {
  constructor(
    @InjectRepository(Infraccion)
    private infraccionRepository: Repository<Infraccion>,
    @InjectRepository(InstitucionInfraccion)
    private institucionInfraccionRepository: Repository<InstitucionInfraccion>,
    private institucionService: InstitucionService,
  ) {}

  create(infraccion: string): Promise<{ message: string }> {
    // Buscamos un registro con ese nombre
    return this.infraccionRepository
      .findOne({ where: { infraccion } })
      .then((existeInfraccion) => {
        // Sacamos error si existe
        if (existeInfraccion)
          throw new ConflictException('Ya existe esta infracción.');
        // Creamos y guardamos un registro
        return this.infraccionRepository.save(
          this.infraccionRepository.create({ infraccion }),
        );
      })
      .then(async (infraccion) => {
        const instituciones = await this.institucionService.findAll();

        // Asociamos la nueva infracción con las instituciones existentes
        for (let i = 0; i < instituciones.length; i++)
          await this.institucionInfraccionRepository.save(
            this.institucionInfraccionRepository.create({
              infraccion,
              institucion: instituciones[i],
            }),
          );
        return { message: 'Se creó correctamente una nueva infracción.' };
      });
  }

  findAll(): Promise<Infraccion[]> {
    return this.infraccionRepository.find({
      where: { id_infraccion: Not(1) },
      order: { infraccion: 'ASC' },
    });
  }

  findAllOperador(operador: Operador): Promise<InstitucionInfraccion[]> {
    return this.institucionInfraccionRepository.find({
      where: {
        infraccion: { id_infraccion: Not(1) },
        institucion: operador.institucion,
      },
      order: { infraccion: { infraccion: 'ASC' } },
    });
  }

  findById(id_institucion_infraccion: number): Promise<InstitucionInfraccion> {
    return this.institucionInfraccionRepository
      .findOne({ where: { id_institucion_infraccion } })
      .then((institucionInfraccion) => {
        if (!institucionInfraccion)
          throw new NotFoundException(
            'No existe este id institución infracción.',
          );
        return institucionInfraccion;
      });
  }

  findSinInfraccion(institucion: Institucion): Promise<InstitucionInfraccion> {
    return this.institucionInfraccionRepository.findOne({
      where: { infraccion: { id_infraccion: 1 }, institucion },
    });
  }

  update(
    admin: Operador,
    attrs: Partial<InstitucionInfraccion>,
  ): Promise<{ message: string }> {
    return this.findById(attrs.id_institucion_infraccion)
      .then((institucionInfraccion) => {
        // Validamos que la institucion infraccion pertenezca a la institución del admin
        if (
          admin.institucion.id_institucion !=
          institucionInfraccion.institucion.id_institucion
        )
          throw new ConflictException(
            'No puedes modificar la información esta infracción porque no pertenece a tu institución.',
          );
        // Asignamos valores enviados al objeto
        Object.assign(institucionInfraccion, attrs);
        // Guardamos cambios
        return this.institucionInfraccionRepository.save(institucionInfraccion);
      })
      .then((_) => ({ message: 'Se guardaron los cambios correctamente.' }));
  }
}
