import {
  ConflictException,
  Injectable,
  NotFoundException,
} from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import {
  FindManyOptions,
  FindOptionsWhere,
  IsNull,
  Not,
  Repository,
} from 'typeorm';
import { Institucion } from './entity/institucion.entity';
import { Operador } from '../operador/entity/operador.entity';

@Injectable()
export class InstitucionService {
  constructor(
    @InjectRepository(Institucion) private repository: Repository<Institucion>,
  ) {}

  // crearInstitucion(id_institucion: number): Institucion {
  //   return this.repository.create({ id_institucion });
  // }

  findAll(): Promise<Institucion[]> {
    return this.repository.find({ select: ['id_institucion'] });
  }

  findById(id_institucion: number): Promise<Institucion> {
    return this.repository
      .findOne({
        select: ['id_institucion', 'logo'],
        where: { id_institucion },
      })
      .then((institucion) => {
        if (!institucion)
          throw new NotFoundException('No existe este id institución.');
        return institucion;
      });
  }

  findFullInfoById(
    admin: Operador,
    id_institucion: number,
  ): Promise<Institucion> {
    return this.repository
      .findOne({ where: { id_institucion } })
      .then((institucion) => {
        if (!institucion)
          throw new NotFoundException('No existe este id institución.');
        // Validamos que la institución pertenezca al admin
        if (
          admin.tipoUsuario.id_tipo_usuario === 3 &&
          admin.institucion.id_institucion != institucion.id_institucion
        )
          throw new ConflictException(
            'No puedes actualizar la información de esta institución porque no perteneces a ella.',
          );
        return institucion;
      });
  }

  findMinInfoAll(activo = false, responsable = false): Promise<Institucion[]> {
    const options: FindManyOptions<Institucion> = {
      select: [
        'id_institucion',
        'activo',
        'correo',
        'institucion',
        'responsable',
      ],
      order: { institucion: 'ASC' },
    };
    const busqueda: FindOptionsWhere<Institucion> = {};

    if (activo) busqueda.activo = activo;
    if (responsable) {
      busqueda.responsable = Not(IsNull());
      options.order = { activo: 'DESC', institucion: 'ASC' };
    }
    options.where = busqueda;
    return this.repository.find(options);
  }

  save(institucion: Institucion): Promise<Institucion> {
    return this.repository.save(institucion);
  }

  update(
    admin: Operador,
    attrs: Partial<Institucion>,
  ): Promise<{ message: string }> {
    return this.findById(attrs.id_institucion)
      .then((institucion) => {
        // Validamos que la institución pertenezca al admin
        if (
          admin.tipoUsuario.id_tipo_usuario === 3 &&
          admin.institucion.id_institucion != institucion.id_institucion
        )
          throw new ConflictException(
            'No puedes modificar la información esta institución porque no te pertenece.',
          );
        // Asignamos valores enviados al objeto
        Object.assign(institucion, attrs);
        // Guardamos cambios
        return this.repository.save(institucion);
      })
      .then((_) => ({ message: 'Se guardaron los cambios correctamente.' }));
  }
}
