import * as moment from 'moment';
import {
  ForbiddenException,
  forwardRef,
  Inject,
  Injectable,
} from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { FindManyOptions, FindOptionsWhere, Repository } from 'typeorm';
import { Modulo } from '../modulo/entity/modulo.entity';
import { ModuloMotivo } from './entity/modulo-motivo.entity';
import { Operador } from '../operador/entity/operador.entity';
import { InstitucionService } from '../institucion/institucion.service';
import { ModuloService } from '../modulo/modulo.service';

@Injectable()
export class ModuloMotivoService {
  constructor(
    @InjectRepository(ModuloMotivo)
    private repository: Repository<ModuloMotivo>,
    private institucionService: InstitucionService,
    @Inject(forwardRef(() => ModuloService))
    private moduloService: ModuloService,
  ) {}

  create(
    modulo: Modulo,
    operador: Operador,
    motivo: string,
    numero_alumnos: number,
    fecha_creacion: string,
  ): Promise<ModuloMotivo> {
    // Cramos y guardamos registro
    return this.repository.save(
      this.repository.create({
        fecha_creacion: moment(fecha_creacion).toDate(),
        motivo,
        numero_alumnos,
        modulo,
        operador,
      }),
    );
  }

  async findAllActividadEspecial(
    admin: Operador,
    id_institucion?: number,
  ): Promise<ModuloMotivo[]> {
    const institucion = id_institucion
      ? await this.institucionService.findById(id_institucion)
      : null;
    const busqueda: FindOptionsWhere<ModuloMotivo> = {};

    if (admin.institucion) busqueda.modulo = { institucion: admin.institucion };
    else if (institucion) busqueda.modulo = { institucion: institucion };
    return this.repository.find({
      join: {
        alias: 'mmo',
        innerJoinAndSelect: { m: 'mmo.modulo', i: 'm.institucion' },
      },
      where: busqueda,
    });
  }

  async findAllByIdModulo(
    admin: Operador,
    id_modulo: number,
    pagina?: number,
  ): Promise<ModuloMotivo[] | [ModuloMotivo[], number]> {
    const modulo = await this.moduloService.findById(id_modulo);
    const options: FindManyOptions<ModuloMotivo> = { where: { modulo } };

    // Validamos que el módulo pertenezca a la institución del admin
    if (
      admin.tipoUsuario.id_tipo_usuario > 2 &&
      modulo.institucion.id_institucion != admin.institucion.id_institucion
    )
      throw new ForbiddenException(
        'No puedes acceder a esta información porque no le pertenece a tu institución.',
      );
    // Si se mandó página significa que requiere paginación
    if (pagina) {
      options.skip = (pagina - 1) * 25;
      options.take = 25;
      return this.repository.findAndCount(options);
    } else return this.repository.find(options);
  }
}
