import {
  BadRequestException,
  ConflictException,
  forwardRef,
  Inject,
  Injectable,
} from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { Repository } from 'typeorm';
import { Modulo } from '../modulo/entity/modulo.entity';
import { Operador } from '../operador/entity/operador.entity';
import { Passcode } from './entity/passcode.entity';
import { Prestamo } from '../prestamo/entity/prestamo.entity';
import { PrestamoService } from '../prestamo/prestamo.service';

@Injectable()
export class PasscodeService {
  constructor(
    @InjectRepository(Passcode) private repository: Repository<Passcode>,
    @Inject(forwardRef(() => PrestamoService))
    private prestamoService: PrestamoService,
  ) {}

  create(prestamo: Prestamo): Promise<Passcode> {
    const aux = ['00000', '0000', '000', '00', '0', ''];
    const carrito = `${prestamo.equipo.carrito.id_carrito < 10 ? '0' : ''}${
      prestamo.equipo.carrito.id_carrito
    }`;
    const random = Math.floor(Math.random() * 99999); // Generamos un número random entre 0-99999
    const randomStr = `${aux[random.toString().length]}${random}`; // Llenamos con los ceros faltantes

    // Creamos y guardamos un registro
    return this.repository
      .save(this.repository.create({ prestamo }))
      .then((passcode) => {
        const id = `${aux[passcode.id_passcode.toString().length - 1]}${
          passcode.id_passcode
        }`; // LLenamos con los ceros faltantes

        // Asignamos valor a passcode
        passcode.passcode = `${carrito}${id}${randomStr}`;
        // Guardamos cambios
        return this.repository.save(passcode);
      });
  }

  devolverEquipo(
    operador: Operador,
    modulo: Modulo,
    passcode: string,
  ): Promise<{ code: number }> {
    return this.findByPasscode(passcode, true)
      .then((passcode) => {
        this.validacionBasica(passcode.prestamo, operador, modulo);
        return this.prestamoService.regresar(
          operador,
          modulo,
          passcode.prestamo,
          1,
        );
      })
      .then((_) => ({ code: 1 }));
  }

  async entregarEquipo(
    operador: Operador,
    modulo: Modulo,
    passcode: string,
    estatus: number,
  ): Promise<{ code: number }> {
    return this.findByPasscode(passcode, false).then((passcode) => {
      this.validacionBasica(passcode.prestamo, operador, modulo);
      if (estatus) {
        if (passcode.abrio != (estatus === 1)) passcode.abrio = estatus === 1;
        return this.prestamoService
          .entregar(operador, modulo, passcode.prestamo)
          .then((_) => this.repository.save(passcode))
          .then((_) => ({ code: 1 }));
      }
    });
  }

  findByPasscode(passcode: string, abrio: boolean): Promise<Passcode> {
    return this.repository
      .findOne({ where: { passcode, abrio } })
      .then((passcode) => {
        if (!passcode)
          throw new BadRequestException({
            code: 8,
            message: `No existe este passcode.`,
          });
        return passcode;
      });
  }

  findByPrestamo(prestamo: Prestamo): Promise<Passcode> {
    return this.repository.findOne({
      select: ['id_passcode', 'passcode'],
      where: { prestamo },
    });
  }

  findPasscode(passcode: string, id_locker: number) {
    return this.findByPasscode(passcode, false).then((passcode) => {
      if (passcode.prestamo.equipo.carrito.id_carrito != id_locker)
        throw new BadRequestException({
          code: 2,
          message: `El id_locker no corresponde`,
        });
      return {
        code: 1,
        door: parseInt(passcode.prestamo.equipo.equipo.substring(1, 3)),
        id_locker,
        passcode: passcode.passcode,
      };
    });
  }

  private validacionBasica(
    prestamo: Prestamo,
    operador: Operador,
    modulo?: Modulo,
  ): void {
    // Validamos si el préstamo ya fue cancelado
    if (
      prestamo.cancelado_operador ||
      prestamo.cancelado_usuario ||
      prestamo.cancelado_repeticion
    )
      throw new ConflictException({
        code: 4,
        message: 'Este préstamo fue cancelado.',
      });
    // Validamos si el préstamo esta inactivo
    if (!prestamo.activo)
      throw new ConflictException({
        code: 7,
        message: 'Este préstamo ya no se encuentra activo.',
      });
    // Validamos que el prestamo pertenezca a la institución del operador
    if (
      operador.institucion.id_institucion !=
      prestamo.equipo.carrito.modulo.institucion.id_institucion
    )
      throw new ConflictException({
        code: 2,
        message: 'Este préstamo no pertenece a esta institución.',
      });
    // Validamos que el prestamo pertenezca al módulo del operador
    if (modulo && prestamo.equipo.carrito.modulo.id_modulo != modulo.id_modulo)
      throw new ConflictException({
        code: 2,
        message: 'Este préstamo no pertenece a tu módulo.',
      });
  }
}
